﻿using System;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Windows;
using Microsoft.Practices.Unity;
using Photoviewer.Common;
using Photoviewer.DomainModel;
using Photoviewer.Modules.Albums.Properties;
using Photoviewer.Modules.Albums.Views;

namespace Photoviewer.Modules.Albums
{
    /// <summary>
    /// Controller for the import process required
    /// for new albums
    /// </summary>
    public class AlbumImportController: ModuleController
    {
        private ImportProgressViewModel _model;
        private ImportProgressWindow _progressWindow;
        private bool _cancelInProgress;

        /// <summary>
        /// Initializes a new instance of the AlbumImportController class.
        /// </summary>
        public AlbumImportController(IUnityContainer container):base(container)
        {
        }

        /// <summary>
        /// Imports a set of photos
        /// </summary>
        /// <param name="sourcePath"></param>
        /// <param name="targetPath"></param>
        public void ImportPhotos(string sourcePath, string targetPath)
        {
            _progressWindow = new ImportProgressWindow();
            _model = new ImportProgressViewModel();

            _progressWindow.Model = _model;

            _progressWindow.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            _progressWindow.OperationCanceled += ((sender, e) => { _cancelInProgress = true; });
            _progressWindow.Show();

            var importAction = new ImportPhotosAction(ImportPhotosCore);
            var callback = new AsyncCallback((ar) => {
                var closeAction = new Action<object>((x) => { _progressWindow.Close(); });
                _progressWindow.Dispatcher.BeginInvoke(closeAction, new object());
            });

            importAction.BeginInvoke(sourcePath, targetPath, callback, null);
        }

        private delegate void ImportPhotosAction(string sourcePath,string targetPath);

        /// <summary>
        /// Handles a single import conflict for a file
        /// </summary>
        /// <param name="info"></param>
        /// <param name="overwrite"></param>
        /// <param name="doCopy"></param>
        /// <param name="shouldReturn"></param>
        private static void HandleImportConflict(FileInfo info, out bool overwrite, out bool doCopy, out bool shouldReturn)
        {
            shouldReturn = false;
            overwrite = false;
            doCopy = true;

            var result = MessageBox.Show(String.Format(Resources.OverwriteExistingPhotoText, info.Name),
                                        Resources.OverwriteExistingPhotoTitle, 
                                        MessageBoxButton.YesNoCancel, MessageBoxImage.Question);

            if (result == MessageBoxResult.Yes)
            {
                overwrite = true;
            }
            else if (result == MessageBoxResult.No)
            {
                doCopy = false;
            }
            else
            {
                shouldReturn = true;
            }
        }

        /// <summary>
        /// Imports photos in the background
        /// </summary>
        /// <param name="sourcePath"></param>
        /// <param name="targetPath"></param>
        private void ImportPhotosCore(string sourcePath, string targetPath)
        {
            var updateStatusAction = new Action<string>((s) => { _model.StatusText = s; });
            var updateProgressAction = new Action<int>((x) => { _model.Progress = x; });

            string[] files = Directory.GetFiles(sourcePath);
            string[] allowedExtensions = new[] { ".bmp", ".jpg", ".gif", ".png", ".jpeg", ".tif" };

            Album targetAlbum = Album.GetAlbum(targetPath);
            int progress = 0;
            int progressStep = 100 / files.Count();

            foreach (string imageFile in files)
            {
                if (_cancelInProgress)
                {
                    return;
                }

                FileInfo info = new FileInfo(imageFile);
                var validExtension = allowedExtensions.Where(extension => String.Compare(
                    info.Extension, extension, true, CultureInfo.InvariantCulture) == 0).Count() != 0;

                if (validExtension)
                {
                    _progressWindow.Dispatcher.BeginInvoke(
                        updateStatusAction, "Copying " + imageFile);

                    string targetFileName = Path.Combine(targetPath, info.Name);
                    bool overwrite = false;
                    bool doCopy = true;

                    if (File.Exists(targetFileName))
                    {
                        bool shouldReturn;
                        HandleImportConflict(info, out overwrite, out doCopy, out shouldReturn);

                        if (shouldReturn)
                            return;
                    }

                    if (doCopy)
                    {
                        File.Copy(imageFile, targetFileName, overwrite);
                    }
                }
                else
                {
                    _progressWindow.Dispatcher.BeginInvoke(
                        updateStatusAction, "Skipping " + imageFile);
                }

                progress += progressStep;

                _progressWindow.Dispatcher.BeginInvoke(updateProgressAction, progress);
            }
        }
    }
}
