﻿using System;
using System.Configuration;
using System.IO;
using System.Windows;
using Microsoft.Practices.Composite.Wpf.Commands;
using Microsoft.Practices.Unity;
using Photoviewer.Common;
using Photoviewer.DomainModel;
using Photoviewer.Modules.Albums.Properties;
using Photoviewer.Modules.Albums.Views;
using Photoviewer.Common.Contract;

namespace Photoviewer.Modules.Albums
{
    /// <summary>
    /// Controller for the albums module
    /// </summary>
    public class AlbumModuleController : ModuleController
    {
        #region Private fields

        private DelegateCommand<Album> _editAlbumDetailsCommand;
        private DelegateCommand<Album> _removeAlbumCommand;
        private DelegateCommand<object> _createAlbumCommand;
        private DelegateCommand<Album> _printAlbumCommand;

        private AlbumsViewPresenter _albumsPresenter;

        #endregion

        #region Public constructors

        /// <summary>
        /// Initializes a new instance of the AlbumModuleController class.
        /// </summary>
        public AlbumModuleController(IUnityContainer container)
            : base(container)
        {
        }

        #endregion

        #region Initialization logic

        /// <summary>
        /// Initializes the controller
        /// </summary>
        public void Initialize()
        {
            InitializeCommands();
            InitializeViews();
        }

        /// <summary>
        /// Initializes the views managed by the controller
        /// </summary>
        private void InitializeViews()
        {
            _albumsPresenter = Container.Resolve<AlbumsViewPresenter>();

            RegionManager.Regions[RegionNames.AlbumSelection].Add(_albumsPresenter.View);
            _albumsPresenter.Initialize();

            // Bind the album selection changed event
            _albumsPresenter.SelectedAlbumChanged += OnSelectedAlbumChanged;
        }

        /// <summary>
        /// Initializes a basic set of commands
        /// </summary>
        private void InitializeCommands()
        {
            _createAlbumCommand = new DelegateCommand<object>(CreateAlbum);
            _editAlbumDetailsCommand = new DelegateCommand<Album>(EditAlbumDetails);
            _removeAlbumCommand = new DelegateCommand<Album>(RemoveAlbum);
            _printAlbumCommand = new DelegateCommand<Album>(PrintAlbum);

            ModuleCommands.CreateAlbumCommand.RegisterCommand(_createAlbumCommand);
            ModuleCommands.EditAlbumDetailsCommand.RegisterCommand(_editAlbumDetailsCommand);
            ModuleCommands.RemoveAlbumCommand.RegisterCommand(_removeAlbumCommand);
            ModuleCommands.PrintAlbumCommand.RegisterCommand(_printAlbumCommand);
        }

        #endregion

        #region Event handlers

        private void OnSelectedAlbumChanged(object sender, object e)
        {
            EventAggregator.GetEvent<ActiveAlbumChangedEvent>().Publish(_albumsPresenter.Model.SelectedAlbum);
        }
        
        #endregion

        #region Use case realizations

        /// <summary>
        /// Prints the album contents
        /// </summary>
        /// <param name="album"></param>
        private void PrintAlbum(Album album)
        {
            IPrintService printService = Container.Resolve<IPrintService>();

            if(printService != null)
            {
                printService.PrintAlbumContents(album);
            }
        }

        /// <summary>
        /// Creates a new album
        /// </summary>
        /// <param name="args"></param>
        private void CreateAlbum(object args)
        {
            ICreateAlbumView view = Container.Resolve<ICreateAlbumView>();
            bool? result = DialogWorkspace.ShowView(view, new DialogOptions { DialogTitle = "Create new album" }, () =>
            {
                if (String.IsNullOrEmpty(view.Title))
                {
                    MessageBox.Show(Resources.TitleRequiredText,
                        Resources.ValidationFailedTitle,
                        MessageBoxButton.OK, MessageBoxImage.Error);

                    return false;
                }

                if (String.IsNullOrEmpty(view.Path))
                {
                    MessageBox.Show(Resources.PathRequiredText,
                        Resources.ValidationFailedTitle,
                        MessageBoxButton.OK, MessageBoxImage.Error);

                    return false;
                }

                return true;
            });

            if (result.HasValue && result.Value)
            {
                DirectoryInfo albumPathInfo = new DirectoryInfo(view.Path);
                DirectoryInfo rootPathInfo = new DirectoryInfo(ConfigurationManager.AppSettings["AlbumsBasePath"]);

                // Check if the album isn't already in the managed folder
                if (albumPathInfo.Parent.FullName != rootPathInfo.FullName)
                {
                    string targetPath = Path.Combine(rootPathInfo.FullName, albumPathInfo.Name);

                    Album newAlbum = Album.NewAlbum(targetPath);
                    newAlbum.Title = view.Title;

                    // Create the new album at the target location and import the photos
                    newAlbum.Save();

                    AlbumImportController importController = Container.Resolve<AlbumImportController>();
                    importController.ImportPhotos(albumPathInfo.FullName, targetPath);
                }
            }
        }

        /// <summary>
        /// Edits the album details
        /// </summary>
        /// <param name="album"></param>
        private void EditAlbumDetails(Album album)
        {
            if (album != null)
            {
                IAlbumDetailsView view = Container.Resolve<IAlbumDetailsView>();
                view.Model = album;

                album.BeginEdit();
                bool? result = DialogWorkspace.ShowView(view,
                    new DialogOptions { DialogTitle = "Edit album details" }, null);

                if (result.HasValue && result.Value)
                {
                    album.ApplyEdit();
                    album.Save();
                }
                else
                {
                    album.CancelEdit();
                }
            }
        }

        /// <summary>
        /// Removes an existing album
        /// </summary>
        /// <param name="album"></param>
        private void RemoveAlbum(Album album)
        {
            if (album != null)
            {
                MessageBoxResult result = MessageBox.Show(
                    Resources.RemoveAlbumConfirmationText,
                    Resources.RemoveAlbumConfirmationTitle,
                    MessageBoxButton.YesNo, MessageBoxImage.Question);

                if (result == MessageBoxResult.Yes)
                {
                    // Delete the album and refresh the view
                    Album.DeleteAlbum(album.Path);
                    _albumsPresenter.Refresh();
                }
            }
        }

        #endregion
    }
}
