﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using Photoviewer.DataAccess.Contract;
using Photoviewer.DataAccess.Implementation.Metadata;

namespace Photoviewer.DataAccess.Implementation
{
    /// <summary>
    /// Implementation for the album data provider
    /// </summary>
    public class AlbumDataProvider: IAlbumDataProvider
    {
        private readonly string _basePath;

        /// <summary>
        /// Initializes a new instance of the AlbumDataProvider class.
        /// </summary>
        /// <param name="basePath"></param>
        public AlbumDataProvider(string basePath)
        {
            _basePath = basePath;
        }

        #region IAlbumDataProvider Members

        /// <summary>
        /// Retrieves all albums found at the base location
        /// </summary>
        /// <returns></returns>
        public Collection<AlbumData> GetAlbums()
        {
            if (!Directory.Exists(_basePath))
                throw new InvalidOperationException("Base directory does not exist");

            return new Collection<AlbumData>(Directory.GetDirectories(_basePath).Select((path) =>
            {
                var data = new AlbumData();
                var metadata = AlbumMetadata.Load(path);

                data.Title = new DirectoryInfo(path).Name;

                if (!String.IsNullOrEmpty(metadata.Title))
                    data.Title = metadata.Title;

                data.Path = path;

                return data;
            }).ToList());
        }

        /// <summary>
        /// Retrieves a single album
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public AlbumData GetAlbumByPath(string path)
        {
            if (!Directory.Exists(_basePath))
                throw new InvalidOperationException("Base directory does not exist");

            string fullPath = Path.Combine(_basePath, path);

            if(!Directory.Exists(fullPath))
                throw new InvalidOperationException("Album directory does not exist");

            AlbumMetadata metadata = AlbumMetadata.Load(fullPath);
            DirectoryInfo info = new DirectoryInfo(fullPath);

            return new AlbumData
                       {
                Title = metadata.Title ?? info.Name,
                Path = fullPath
            };
        }

        /// <summary>
        /// Creates a new album
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public AlbumData CreateAlbum(AlbumData data)
        {
            string albumPath = Path.IsPathRooted(data.Path) ? data.Path : Path.Combine(_basePath, data.Path);

            if(String.Compare(new DirectoryInfo(albumPath).Parent.FullName,_basePath,true) != 0)
                throw new ArgumentException("The provided path is not a direct child of the base path");

            if (!Directory.Exists(albumPath))
            {
                Directory.CreateDirectory(albumPath);
            }

            AlbumMetadata metadata = new AlbumMetadata();
            metadata.Title = data.Title;

            metadata.Save(albumPath);

            return data;
        }

        /// <summary>
        /// Updates album metadata
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public AlbumData UpdateAlbum(AlbumData data)
        {
            string albumPath = Path.IsPathRooted(data.Path) ? data.Path : Path.Combine(_basePath, data.Path);

            if (!Directory.Exists(albumPath))
                throw new ArgumentException("Album path does not exist");

            AlbumMetadata metadata = AlbumMetadata.Load(albumPath);
            metadata.Title = data.Title;

            metadata.Save(albumPath);

            return data;
        }

        /// <summary>
        /// Deletes an existing album
        /// </summary>
        /// <param name="path"></param>
        public void DeleteAlbum(string path)
        {
            string albumPath = Path.IsPathRooted(path) ? path : Path.Combine(_basePath, path);

            if (!Directory.Exists(albumPath))
                throw new ArgumentException("Album directory does not exist");

            string[] files = Directory.GetFiles(albumPath);

            foreach (string filePath in files)
            {
                File.Delete(filePath);
            }

            Directory.Delete(albumPath);
        }

        #endregion
    }
}
