﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Xml.Serialization;

namespace Photoviewer.DataAccess.Implementation.Metadata
{
    /// <summary>
    /// Represents album metadata
    /// </summary>
    [XmlRoot(ElementName = "AlbumMetadata")]
    public class AlbumMetadata
    {
        /// <summary>
        /// Initializes a new instance of the AlbumMetadata class.
        /// </summary>
        public AlbumMetadata()
        {
            Photos = new Collection<Photo>();
        }

        #region Public properties

        /// <summary>
        /// Gets or sets the title of the album
        /// </summary>
        [XmlAttribute(AttributeName = "title")]
        public string Title { get; set; }

        /// <summary>
        /// Gets or sets the photos in the album
        /// </summary>
        [XmlArray(ElementName = "Photos")]
        public Collection<Photo> Photos { get; set; }

        #endregion

        #region Public Methods

        /// <summary>
        /// Saves the album metadata
        /// </summary>
        /// <param name="albumPath"></param>
        public void Save(string albumPath)
        {
            string metadataPath = Path.Combine(albumPath, ".metadata.xml");
            FileStream outputStream = null;

            try
            {
                if (File.Exists(metadataPath))
                {
                    outputStream = new FileStream(metadataPath, FileMode.Truncate, FileAccess.Write);
                }
                else
                {
                    outputStream = File.Create(metadataPath);
                }

                XmlSerializer serializer = new XmlSerializer(typeof(AlbumMetadata));
                serializer.Serialize(outputStream, this);
            }
            catch (Exception ex)
            {

            }
            finally
            {
                if(outputStream != null)
                    outputStream.Close();

                FileInfo info = new FileInfo(metadataPath);
                info.Attributes |= FileAttributes.Hidden;
            }
        }

        #endregion

        #region Factory methods

        /// <summary>
        /// Loads metadata for an album
        /// </summary>
        /// <param name="albumPath">Path to the album</param>
        /// <returns>Returns the loaded album metadata</returns>
        public static AlbumMetadata Load(string albumPath)
        {
            AlbumMetadata result = null;
            string metadataPath = Path.Combine(albumPath, ".metadata.xml");

            if (!File.Exists(metadataPath))
            {
                result = new AlbumMetadata();
            }
            else
            {
                using (Stream inputStream = File.OpenRead(metadataPath))
                {
                    XmlSerializer serializer = new XmlSerializer(typeof(AlbumMetadata));
                    result = (AlbumMetadata)serializer.Deserialize(inputStream);
                }
            }

            return result;
        }

        #endregion
    }
}
