﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using Photoviewer.DataAccess.Contract;
using Photoviewer.DataAccess.Implementation.Metadata;

namespace Photoviewer.DataAccess.Implementation
{
    /// <summary>
    /// Implementation of the photo data provider
    /// </summary>
    public class PhotoDataProvider : IPhotoDataProvider
    {

        private readonly string _basePath;

        /// <summary>
        /// Contains all supported file types
        /// </summary>
        private readonly string[] _supportedExtensions = new[] {
            ".jpg",".gif",".bmp",".png",".tif"
        };

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the PhotoDataProvider class.
        /// </summary>
        /// <param name="basePath"></param>
        public PhotoDataProvider(string basePath)
        {
            _basePath = basePath;
        }

        #endregion

        #region IPhotoDataProvider Members

        /// <summary>
        /// Gets raw image data from disk
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public Stream GetImageData(string path)
        {
            if (string.IsNullOrEmpty(path))
                throw new ArgumentNullException("path");

            if (!File.Exists(path))
                throw new PhotoNotFoundException("Path does not exist", path);

            return File.OpenRead(path);
        }

        /// <summary>
        /// Gets a single photo
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public PhotoData GetPhotoByPath(string path)
        {
            PhotoData result = null;
            FileInfo info = new FileInfo(path);

            if (!info.Exists)
                throw new ArgumentException("The requested photo does not exist", "path");

            AlbumMetadata metadata = AlbumMetadata.Load(info.Directory.FullName);
            var foundPhotoMetadata = metadata.Photos.Where(p => string.Compare(p.Path, path, true) == 0);

            if (foundPhotoMetadata.Count() > 0)
            {
                var photo = foundPhotoMetadata.First();

                result = new PhotoData
                             {
                    Title = photo.Title,
                    Tags = photo.Tags,
                    Path = photo.Path,
                    AlbumPath = info.Directory.FullName
                };
            }
            else
            {
                result = new PhotoData
                             {
                    Title = info.Name,
                    Path = path,
                    AlbumPath = info.Directory.FullName
                };
            }

            return result;
        }

        /// <summary>
        /// Creates new photo metadata
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public PhotoData CreatePhotoMetadata(PhotoData data)
        {
            string photoPath = Path.IsPathRooted(data.Path) ? data.Path : Path.Combine(data.AlbumPath, data.Path);

            AlbumMetadata metadata = AlbumMetadata.Load(data.AlbumPath);
            var foundPhotos = metadata.Photos.Where(p => p.Path == photoPath);

            if (!File.Exists(photoPath))
                throw new PhotoNotFoundException("The photo was not found on disk", data.Path);

            if (foundPhotos.Count() > 0)
                throw new InvalidOperationException("Photo metadata already exists");

            Photo photoMetadata = new Photo { Path = photoPath, Tags = data.Tags, Title = data.Title };

            // Save the new metadata
            metadata.Photos.Add(photoMetadata);
            metadata.Save(data.AlbumPath);

            // Update the original data with the new path
            data.Path = photoPath;

            return data;
        }

        /// <summary>
        /// Updates existing photo metadata
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public PhotoData UpdatePhotoMetadata(PhotoData data)
        {
            if (!File.Exists(data.Path))
                throw new PhotoNotFoundException("The photo does not exist on disk", data.Path);

            AlbumMetadata metadata = AlbumMetadata.Load(data.AlbumPath);
            string photoPath = Path.IsPathRooted(data.Path) ? data.Path : Path.GetFullPath(Path.Combine(data.AlbumPath, data.Path));
            var foundPhotos = from p in metadata.Photos where p.Path == photoPath select p;

            if (foundPhotos.Count() == 0)
            {
                CreatePhotoMetadata(data);
                return data;
            }

            // Update the metadata
            var photoMetadata = foundPhotos.First();
            photoMetadata.Title = data.Title;
            photoMetadata.Tags = data.Tags;

            metadata.Save(data.AlbumPath);

            // Save the fixed path
            data.Path = photoPath;

            return data;
        }

        /// <summary>
        /// Deletes a single photo
        /// </summary>
        /// <param name="path"></param>
        public void DeletePhoto(string path)
        {
            if (!File.Exists(path))
                throw new PhotoNotFoundException("The photo was not found on disk", path);

            FileInfo info = new FileInfo(path);
            AlbumMetadata metadata = AlbumMetadata.Load(info.Directory.FullName);

            var foundPhotos = metadata.Photos.Where(p => p.Path == info.FullName);

            // The application shouldn't care if the metadata does exist at this moment
            // We are going to remove it anyway
            if (foundPhotos.Count() > 0)
            {
                metadata.Photos.Remove(foundPhotos.First());
                metadata.Save(info.Directory.FullName);
            }

            // Delete the file from disk
            File.Delete(path);
        }

        /// <summary>
        /// Retrieves all photos from a single album
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public Collection<PhotoData> GetPhotosByAlbumPath(string path)
        {
            if (!Directory.Exists(path))
                throw new AlbumNotFoundException("Album was not found on disk", path);

            AlbumMetadata metadata = AlbumMetadata.Load(path);

            return new Collection<PhotoData>(Directory.GetFiles(path)
                .Where(filePath =>
                {
                    FileInfo info = new FileInfo(filePath);
                    return _supportedExtensions.Where(ext => ext.ToLower() == info.Extension.ToLower()).Count() > 0;
                })
                .Select(filePath =>
                {
                    var foundPhotos = from p in metadata.Photos where p.Path == filePath select p;
                    var data = new PhotoData { Path = filePath, AlbumPath = path };

                    if (foundPhotos.Count() > 0)
                    {
                        var photoMetadata = foundPhotos.First();
                        data.Title = photoMetadata.Title;
                        data.Tags = photoMetadata.Tags;
                    }
                    else
                    {
                        FileInfo info = new FileInfo(filePath);
                        data.Title = info.Name;
                    }

                    return data;
                }).ToList());
        }

        public Collection<PhotoData> GetPhotosByTag(string tagName)
        {
            Collection<PhotoData> result = new Collection<PhotoData>();

            string[] albumFolders = Directory.GetDirectories(_basePath);

            foreach (string albumFolder in albumFolders)
            {
                AlbumMetadata metadata = AlbumMetadata.Load(albumFolder);
                var foundPhotos = metadata.Photos.Where(
                    photo =>
                    {
                        foreach (string tag in photo.Tags)
                        {
                            if (tag.ToLower() == tagName.ToLower()) return true;
                        }
                        return false;
                    });

                if (foundPhotos.Count() > 0)
                {
                    var photos =foundPhotos.Select(
                    photo => new PhotoData
                                 {
                        AlbumPath = albumFolder,
                        Title = photo.Title,
                        Tags = photo.Tags,
                        Path = photo.Path
                    });

                    foreach (var photo in photos)
                    {
                        result.Add(photo);
                    }
                }
            }

            return result;
        }

        #endregion
    }
}
