﻿using System;
using Csla;
using Photoviewer.DataAccess.Contract;

namespace Photoviewer.DomainModel
{
    /// <summary>
    /// Represents an album for several photos
    /// </summary>
    [Serializable]
    public class Album : BusinessBase<Album>
    {
        #region Business Properties

        private static readonly PropertyInfo<string> PathProperty = RegisterProperty(typeof(Album), new PropertyInfo<string>("Path", "Path"));
        private static readonly PropertyInfo<string> TitleProperty = RegisterProperty(typeof(Album), new PropertyInfo<string>("Title", "Title"));

        private PhotoCollection _photos;

        /// <summary>
        /// Gets the title of the album
        /// </summary>
        public string Title
        {
            get { return GetProperty<string, string>(TitleProperty); }
            set { SetProperty<string, string>(TitleProperty, value); }
        }

        /// <summary>
        /// Gets the path of the album
        /// </summary>
        public string Path
        {
            get { return GetProperty(PathProperty); }
        }

        /// <summary>
        /// Gets the photos contained in the album
        /// </summary>
        public PhotoCollection Photos
        {
            get
            {
                if(_photos == null)
                    _photos = PhotoCollection.GetPhotos(this);

                return _photos;
            }
        }

        #endregion

        #region Factory Methods

        /// <summary>
        /// Creates a new album
        /// </summary>
        /// <returns></returns>
        public static Album NewAlbum(string path)
        {
            if(path == null)
                throw new ArgumentNullException("path");

            return DataPortal.Create<Album>(new SingleCriteria<Album, string>(path));
        }

        /// <summary>
        /// Retrieves a single album
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static Album GetAlbum(string path)
        {
            return DataPortal.Fetch<Album>(new SingleCriteria<Album, string>(path));
        }

        /// <summary>
        /// Gets an album containing all the photos with the specified tag
        /// </summary>
        /// <param name="tagName"></param>
        /// <returns></returns>
        public static Album GetTagsAlbum(string tagName)
        {
            return DataPortal.Fetch<Album>(new AlbumTagCriteria(tagName));
        }

        /// <summary>
        /// Retrieves a single album
        /// </summary>
        /// <param name="album"></param>
        /// <returns></returns>
        public static Album GetAlbum(AlbumData album)
        {
            return DataPortal.Fetch<Album>(album);
        }

        /// <summary>
        /// Deletes a single album
        /// </summary>
        /// <param name="path"></param>
        public static void DeleteAlbum(string path)
        {
            if(String.IsNullOrEmpty(path))
                throw new ArgumentNullException("path");

            DataPortal.Delete(new SingleCriteria<Album, string>(path));
        }

        /// <summary>
        /// Initializes a new instance of <see cref="Album"/>
        /// </summary>
        private Album()
        {  /* Require use of factory methods */ }

        #endregion

        #region Data Access

        /// <summary>
        /// Initializes the default settings for the album
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Create(SingleCriteria<Album,string> criteria)
        {
            LoadProperty<string>(PathProperty, criteria.Value);
        }

        /// <summary>
        /// Fetches the metadata of the album
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Fetch(SingleCriteria<Album, string> criteria)
        {
            var provider = ModelContext.Container.Resolve<IAlbumDataProvider>();
            AlbumData data = provider.GetAlbumByPath(criteria.Value);

            LoadObjectData(data);
        }

        /// <summary>
        /// Fetches the metadata of the album
        /// </summary>
        /// <param name="data"></param>
        private void DataPortal_Fetch(AlbumData data)
        {
            LoadObjectData(data);
        }

        /// <summary>
        /// Fetches the contents of a tag album
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Fetch(AlbumTagCriteria criteria)
        {
            LoadProperty<string>(TitleProperty, criteria.TagName);
            LoadProperty<string>(PathProperty, criteria.TagName);

            _photos = PhotoCollection.GetPhotosByTag(this,criteria.TagName);
        }

        /// <summary>
        /// Creates the album
        /// </summary>
        protected override void DataPortal_Insert()
        {
            var provider = ModelContext.Container.Resolve<IAlbumDataProvider>();
            AlbumData data = provider.CreateAlbum(ExtractObjectData());

            LoadObjectData(data);
        }

        /// <summary>
        /// Updates the album metadata
        /// </summary>
        protected override void DataPortal_Update()
        {
            var provider = ModelContext.Container.Resolve<IAlbumDataProvider>();
            AlbumData data = provider.UpdateAlbum(ExtractObjectData());

            LoadObjectData(data);
        }

        /// <summary>
        /// Marks the album for deletion
        /// </summary>
        protected override void DataPortal_DeleteSelf()
        {
            DataPortal_Delete(new SingleCriteria<Album, string>(GetProperty(PathProperty)));
        }

        /// <summary>
        /// Deletes the album
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Delete(SingleCriteria<Album, string> criteria)
        {
            var provider = ModelContext.Container.Resolve<IAlbumDataProvider>();
            provider.DeleteAlbum(criteria.Value);

            MarkDeleted();
        }

        /// <summary>
        /// Loads raw object data
        /// </summary>
        /// <param name="data"></param>
        private void LoadObjectData(AlbumData data)
        {
            LoadProperty<string>(PathProperty, data.Path);
            LoadProperty<string>(TitleProperty, data.Title);
        }

        /// <summary>
        /// Extracts raw object data
        /// </summary>
        /// <returns></returns>
        private AlbumData ExtractObjectData()
        {
            return new AlbumData
                       {
                Title = ReadProperty(TitleProperty),
                Path = ReadProperty(PathProperty)
            };
        }

        #endregion

        #region Object comparison

        public override bool Equals(object obj)
        {
            Album other = obj as Album;

            if (other != null)
            {
                return other.Path == Path;
            }

            return false;
        }

        public override int GetHashCode()
        {
            return Path.GetHashCode();
        }

        #endregion

        #region Criteria

        [Serializable]
        private class AlbumTagCriteria
        {
            readonly string _tagName;

            /// <summary>
            /// Initializes a new instance of the AlbumTagCriteria class.
            /// </summary>
            /// <param name="tagName"></param>
            public AlbumTagCriteria(string tagName)
            {
                _tagName = tagName;
            }

            public string TagName
            {
                get
                {
                    return _tagName;
                }
            }
        }
        
        #endregion
    }
}
