﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using Csla;
using Photoviewer.DataAccess.Contract;

namespace Photoviewer.DomainModel
{
    /// <summary>
    /// Represents a photo
    /// </summary>
    [Serializable]
    public class Photo : BusinessBase<Photo>
    {
        #region Business properties

        private static readonly PropertyInfo<string> PathProperty = RegisterProperty(typeof(Photo), new PropertyInfo<string>("Path", "Path"));
        private static readonly PropertyInfo<string> TitleProperty = RegisterProperty(typeof(Photo), new PropertyInfo<string>("Title", "Title"));
        private static readonly PropertyInfo<Collection<string>> TagsProperty = RegisterProperty(typeof(Photo), new PropertyInfo<Collection<string>>("Tags", "Tags"));
        private static readonly PropertyInfo<Album> AlbumProperty = RegisterProperty(typeof(Photo), new PropertyInfo<Album>("Album", "Album"));

        /// <summary>
        /// Gets the album the photo belongs to
        /// </summary>
        public Album Album
        {
            get { return GetProperty(AlbumProperty); }
        }

        /// <summary>
        /// Gets the date the photo was created
        /// </summary>
        public DateTime DateCreated
        {
            get
            {
                FileInfo info = new FileInfo(Path);
                return info.CreationTime;
            }
        }

        /// <summary>
        /// Gets or sets the tags of the photo
        /// </summary>
        public Collection<string> Tags
        {
            get { return GetProperty(TagsProperty); }
            set { SetProperty<Collection<string>>(TagsProperty, value); }
        }

        /// <summary>
        /// Gets or sets the title of the photo
        /// </summary>
        public string Title
        {
            get { return GetProperty(TitleProperty); }
            set { SetProperty<string>(TitleProperty, value); }
        }

        /// <summary>
        /// Gets the path of the photo
        /// </summary>
        public string Path
        {
            get { return GetProperty(PathProperty); }
        }

        #endregion

        #region Business Methods

        /// <summary>
        /// Loads the actual photo data
        /// </summary>
        /// <returns></returns>
        public Stream Load()
        {
            var provider = ModelContext.Container.Resolve<IPhotoDataProvider>();
            Stream dataStream = provider.GetImageData(Path);

            return dataStream;
        }

        #endregion

        #region Factory Methods

        /// <summary>
        /// Creates a new photo
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static Photo NewPhoto(Album album,string path)
        {
            return DataPortal.Create<Photo>(new CreatePhotoCriteria { Album = album, Path = path } );
        }

        /// <summary>
        /// Gets a single photo
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static Photo GetPhoto(string path)
        {
            return DataPortal.Fetch<Photo>(new SingleCriteria<Photo, string>(path));
        }

        /// <summary>
        /// Gets a single photo as part of retrieving the contents of an album
        /// </summary>
        /// <param name="album"></param>
        /// <param name="photo"></param>
        /// <returns></returns>
        public static Photo GetPhoto(Album album, PhotoData photo)
        {
            return DataPortal.Fetch<Photo>(new FetchPhotoCriteria { Album = album, Data = photo });
        }

        /// <summary>
        /// Deletes a single photo
        /// </summary>
        /// <param name="path"></param>
        public static void DeletePhoto(string path)
        {
            if(String.IsNullOrEmpty(path))
                throw new ArgumentNullException("path");

            DataPortal.Delete(new SingleCriteria<Photo, string>(path));
        }

        /// <summary>
        /// Initializes a new instance of Photo
        /// </summary>
        private Photo()
        {  /* Require use of factory methods */ }

        #endregion

        #region Data Access

        /// <summary>
        /// Initializes the default values for the photo
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Create(CreatePhotoCriteria criteria)
        {
            LoadProperty<string>(PathProperty, criteria.Path);
            LoadProperty<Album>(AlbumProperty, criteria.Album);
            LoadProperty<Collection<string>>(TagsProperty, new Collection<string>());
        }

        /// <summary>
        /// Fetches the metadata of the photo
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Fetch(SingleCriteria<Photo, string> criteria)
        {
            var provider = ModelContext.Container.Resolve<IPhotoDataProvider>();
            PhotoData data = provider.GetPhotoByPath(criteria.Value);
            
            LoadObjectData(data);
            LoadProperty<Album>(AlbumProperty, Album.GetAlbum(data.AlbumPath));
        }

        /// <summary>
        /// Fetches the metadata of the photo
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Fetch(FetchPhotoCriteria criteria)
        {
            LoadObjectData(criteria.Data);
            LoadProperty<Album>(AlbumProperty, criteria.Album);
        }

        /// <summary>
        /// Creates new photo metadata
        /// </summary>
        protected override void DataPortal_Insert()
        {
            var provider = ModelContext.Container.Resolve<IPhotoDataProvider>();
            var data = ExtractObjectData();

            data = provider.CreatePhotoMetadata(data);
            LoadObjectData(data);
        }

        /// <summary>
        /// Updates existing photo metadata
        /// </summary>
        protected override void DataPortal_Update()
        {
            var provider = ModelContext.Container.Resolve<IPhotoDataProvider>();
            var data = ExtractObjectData();

            data = provider.UpdatePhotoMetadata(data);
            LoadObjectData(data);
        }

        /// <summary>
        /// Marks the photo for deletion
        /// </summary>
        protected override void DataPortal_DeleteSelf()
        {
            DataPortal_Delete(new SingleCriteria<Photo, string>(GetProperty(PathProperty)));
        }

        /// <summary>
        /// Deletes the photo metadata
        /// </summary>
        /// <param name="criteria"></param>
        private void DataPortal_Delete(SingleCriteria<Photo, string> criteria)
        {
            var provider = ModelContext.Container.Resolve<IPhotoDataProvider>();
            provider.DeletePhoto(criteria.Value);
        }

        /// <summary>
        /// Loads raw object data
        /// </summary>
        /// <param name="data"></param>
        private void LoadObjectData(PhotoData data)
        {
            LoadProperty<string>(TitleProperty, data.Title);
            LoadProperty<string>(PathProperty, data.Path);
            LoadProperty<Collection<string>>(TagsProperty, data.Tags);
        }

        /// <summary>
        /// Extracts raw object data
        /// </summary>
        /// <returns></returns>
        private PhotoData ExtractObjectData()
        {
            PhotoData data = new PhotoData
            {
                Title = ReadProperty(TitleProperty),
                Tags = ReadProperty(TagsProperty),
                Path = ReadProperty(PathProperty),
                AlbumPath = Album.Path
            };

            return data;
        }

        #endregion

        #region Criteria Classes

        /// <summary>
        /// Criteria required for creating a new photo
        /// </summary>
        [Serializable]
        private class CreatePhotoCriteria
        {
            public string Path { get; set; }
            public Album Album { get; set; }
        }

        /// <summary>
        /// Criteria required to retrieve a photo
        /// as part of a complete album
        /// </summary>
        [Serializable]
        private class FetchPhotoCriteria
        {
            public Album Album { get; set; }
            public PhotoData Data { get; set; }
        }

        #endregion

        #region Object comparison

        public override bool Equals(object obj)
        {
            Photo other = obj as Photo;

            if (other != null)
            {
                return other.Path == Path;
            }

            return false;
        }

        public override int GetHashCode()
        {
            return Path.GetHashCode();
        }

        #endregion
    }
}
