﻿using System;
using System.Collections.ObjectModel;
using Csla;
using Photoviewer.DataAccess.Contract;

namespace Photoviewer.DomainModel
{
    /// <summary>
    /// Contains a set of photos
    /// </summary>
    [Serializable]
    public class PhotoCollection: ReadOnlyListBase<PhotoCollection,Photo>
    {
        #region Factory methods

        /// <summary>
        /// Gets all photos from an album
        /// </summary>
        /// <param name="album"></param>
        /// <returns></returns>
        public static PhotoCollection GetPhotos(Album album)
        {
            return DataPortal.Fetch<PhotoCollection>(album);
        }

        public static PhotoCollection GetPhotosByTag(Album album,string tagName)
        {
            return DataPortal.Fetch<PhotoCollection>(new TaggedAlbumCriteria(album,tagName));
        }

        #endregion

        #region Data access methods
        
        /// <summary>
        /// Fetches the contents of the collection
        /// </summary>
        /// <param name="album"></param>
        private void DataPortal_Fetch(Album album)
        {
            var provider = ModelContext.Container.Resolve<IPhotoDataProvider>();
            Collection<PhotoData> data = provider.GetPhotosByAlbumPath(album.Path);

            IsReadOnly = false;
            RaiseListChangedEvents = false;

            foreach (var photo in data)
            {
                Add(Photo.GetPhoto(album, photo));
            }

            IsReadOnly = true;
            RaiseListChangedEvents = true;
        }

        /// <summary>
        /// Fetches the contents of the collection
        /// </summary>
        /// <param name="album"></param>
        private void DataPortal_Fetch(TaggedAlbumCriteria criteria)
        {
            var provider = ModelContext.Container.Resolve<IPhotoDataProvider>();
            Collection<PhotoData> data = provider.GetPhotosByTag(criteria.TagName);

            IsReadOnly = false;
            RaiseListChangedEvents = false;

            foreach (var photo in data)
            {
                Add(Photo.GetPhoto(criteria.Album, photo));
            }

            IsReadOnly = true;
            RaiseListChangedEvents = true;
        }

        #endregion

        public class TaggedAlbumCriteria
        {
            readonly Album _album;
            readonly string _tagName;
            
            /// <summary>
            /// Initializes a new instance of the TaggedAlbumCriteria class.
            /// </summary>
            /// <param name="album"></param>
            /// <param name="tagName"></param>
            public TaggedAlbumCriteria(Album album, string tagName)
            {
                _album = album;
                _tagName = tagName;
            }

            public Album Album
            {
                get
                {
                    return _album;
                }
            }

            public string TagName
            {
                get
                {
                    return _tagName;
                }
            }
            
            
        }
    }
}
