﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace Photoviewer.DomainModel
{
    /// <summary>
    /// Represents a tag cloud containing all the tags
    /// defined for all albums
    /// </summary>
    public class TagCloud: INotifyPropertyChanged
    {
        private readonly Collection<Tag> _tags;

        /// <summary>
        /// Initializes a new instance of the TagCloud class.
        /// </summary>
        public TagCloud()
        {
            _tags = new Collection<Tag>();
        }

        /// <summary>
        /// Refreshes the contents of the tag cloud
        /// </summary>
        public void Refresh()
        {
            AlbumCollection albums = AlbumCollection.GetAlbums();

            GatherRawTagData(albums);
            CalculateDeviations();

            OnPropertyChanged(new PropertyChangedEventArgs("Tags"));
        }

        /// <summary>
        /// Gathers the raw tag cloud data
        /// </summary>
        /// <param name="albums"></param>
        private void GatherRawTagData(AlbumCollection albums)
        {
            Dictionary<string, Tag> tags = new Dictionary<string, Tag>();

            // Gather the tags and record the occurrences of each tag
            foreach (Album album in albums)
            {
                foreach (Photo photo in album.Photos)
                {
                    foreach (string tagValue in photo.Tags)
                    {
                        Tag tag = null;
                        string capitalizedTagName = CapitalizeTag(tagValue);

                        if (!tags.TryGetValue(capitalizedTagName, out tag))
                        {
                            tag = new Tag();
                            tag.Name = capitalizedTagName;

                            tags.Add(capitalizedTagName, tag);
                        }

                        tag.Occurrences++;
                    }
                }
            }

            // Move the tags over to the tags collection
            _tags.Clear();

            foreach (Tag tag in tags.Values)
            {
                _tags.Add(tag);
            }
        }

        /// <summary>
        /// Calculates how much each tag is actually different from the 
        /// average tag in terms of occurrences
        /// </summary>
        private void CalculateDeviations()
        {
            if (_tags.Count > 0)
            {
                double max = _tags.Select(tag => tag.Occurrences).Max();

                foreach (Tag tag in _tags)
                {
                    tag.Deviation = (tag.Occurrences / max) * 100;
                }
            }
        }

        /// <summary>
        /// Helper method to capitalize the first letter of a tag
        /// </summary>
        /// <param name="tagName"></param>
        /// <returns></returns>
        private string CapitalizeTag(string tagName)
        {
            StringBuilder result = new StringBuilder();
            result.Append(tagName.Substring(0, 1).ToUpper());
            result.Append(tagName.Substring(1));

            return result.ToString();
        }

        /// <summary>
        /// Gets the tags in the tag cloud
        /// </summary>
        public Collection<Tag> Tags
        {
            get { return _tags; }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #region OnPropertyChanged
        /// <summary>
        /// Triggers the PropertyChanged event.
        /// </summary>
        public virtual void OnPropertyChanged(PropertyChangedEventArgs ea)
        {
            if (PropertyChanged != null)
                PropertyChanged(null/*this*/, ea);
        }
        #endregion

        #endregion
    }
}
