﻿using System;
using System.Windows;
using Microsoft.Practices.Composite.Wpf.Events;
using Microsoft.Practices.Unity;
using Photoviewer.Common;
using Photoviewer.DomainModel;
using Photoviewer.Modules.Photos.Properties;
using Photoviewer.Modules.Photos.Views;

namespace Photoviewer.Modules.Photos
{
    /// <summary>
    /// Controller for the photo module
    /// </summary>
    public class PhotoModuleController : ModuleController
    {
        #region Private fields

        private PhotosViewPresenter _photosPresenter;
        private PhotoDetailsPresenter _detailsPresenter;
        private IPhotoSummaryView _summaryView;
        private CollageViewPresenter _collagePresenter;

        #endregion

        #region Public constructors

        /// <summary>
        /// Initializes a new instance of the PhotoModuleController class.
        /// </summary>
        public PhotoModuleController(IUnityContainer container)
            : base(container)
        {

        }

        #endregion

        #region Initialization logic

        /// <summary>
        /// Initializes the controller for the first time
        /// </summary>
        public void Initialize()
        {
            InitializeViews();
            InitializeEventSubscriptions();
        }

        /// <summary>
        /// Initializes the required views
        /// </summary>
        private void InitializeViews()
        {
            _photosPresenter = Container.Resolve<PhotosViewPresenter>();
            _summaryView = Container.Resolve<IPhotoSummaryView>();
            _collagePresenter = Container.Resolve<CollageViewPresenter>();

            _photosPresenter.PhotoDetailsRequested += OnPhotoDetailsRequested;
            _photosPresenter.SelectedPhotoChanged += OnSelectedPhotoChanged;

            _collagePresenter.PhotoDetailsRequested += OnPhotoDetailsRequested;
            _collagePresenter.SelectedPhotoChanged += OnSelectedPhotoChanged;

            RegionManager.Regions[RegionNames.AlbumContents].Add(_photosPresenter.View);
            RegionManager.Regions[RegionNames.Summary].Add(_summaryView);
            RegionManager.Regions[RegionNames.AlbumContents].Add(_collagePresenter.View);
        }

        

        /// <summary>
        /// Initializes the event subscriptions required by the controller
        /// </summary>
        private void InitializeEventSubscriptions()
        {
            ActiveAlbumChangedEvent activeAlbumChangedEvent = EventAggregator.GetEvent<ActiveAlbumChangedEvent>();
            activeAlbumChangedEvent.Subscribe(OnActiveAlbumChanged, ThreadOption.UIThread, true);
        }

        #endregion

        #region Event subscriptions

        /// <summary>
        /// Handles the SelectedPhotoChanged event of the photos presenter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void OnSelectedPhotoChanged(object sender, EventArgs e)
        {
            IPhotoSelector selector = sender as IPhotoSelector;

            if(selector != null)
            {
                _summaryView.Model = selector.SelectedPhoto;    
            }
            
        }

        /// <summary>
        /// Handles the PhotoDetailsRequested event of the photos presenter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void OnPhotoDetailsRequested(object sender, EventArgs e)
        {
            IPhotoSelector selector = sender as IPhotoSelector;

            if (selector != null)
            {
                _detailsPresenter = Container.Resolve<PhotoDetailsPresenter>();

                _detailsPresenter.PhotoRemovalRequested += OnPhotoRemovalRequested;
                _detailsPresenter.ShowPhotoDetails(selector.SelectedPhoto);
            }
        }

        /// <summary>
        /// Subscription to the <see cref="Photoviewer.Common.ActiveAlbumChangedEvent"/> event
        /// </summary>
        /// <param name="album"></param>
        private void OnActiveAlbumChanged(Album album)
        {
            _photosPresenter.ShowAlbumContents(album);
            _collagePresenter.ShowAlbumContents(album);
        }

        /// <summary>
        /// Handles the PhotoRemovalRequested event of the photo details presenter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnPhotoRemovalRequested(object sender, EventArgs e)
        {
            MessageBoxResult result = MessageBox.Show(Resources.RemovePhotoText, 
                Resources.RemovePhotoTitle, MessageBoxButton.YesNo, MessageBoxImage.Question);

            if(result == MessageBoxResult.Yes)
            {
                // Mark the photo for deletion and process the changes
                _detailsPresenter.CurrentPhoto.Original.Delete();
                _detailsPresenter.CurrentPhoto.Original.Save();
            }
        }

        #endregion
    }
}
