﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using Photoviewer.Common;
using Photoviewer.DomainModel;

namespace Photoviewer.Modules.Photos.Views
{
    public class PhotoViewModel : INotifyPropertyChanged
    {
        private readonly Photo _original;
        private ImageSource _thumbnail;
        private ImageSource _fullImage;
        private bool _retrievingThumbnail;
        private bool _retrievingPhoto;
        private double _zoomFactor;
        private ICommand _showNextPhotoCommand;
        private ICommand _showPreviousPhotoCommand;
        private ICommand _printPhotoCommand;
        private ICommand _saveChangesCommand;
        private ICommand _deletePhotoCommand;
        private ICommand _closeDetailsCommand;

        /// <summary>
        /// Initializes a new instance of the PhotoViewModel class.
        /// </summary>
        /// <param name="original"></param>
        public PhotoViewModel(Photo original)
        {
            _original = original;
            _zoomFactor = 1.0;
        }

        #region Commands

        /// <summary>
        /// Gets or sets the command to close the details view
        /// </summary>
        public ICommand CloseDetailsCommand
        {
            get
            {
                return _closeDetailsCommand;
            }
            set
            {
                _closeDetailsCommand = value;
                OnPropertyChanged(new PropertyChangedEventArgs("CloseDetailsCommand"));
            }
        }

        /// <summary>
        /// Gets or sets the command to print the photo
        /// </summary>
        public ICommand PrintPhotoCommand
        {
            get
            {
                return _printPhotoCommand;
            }
            set
            {
                _printPhotoCommand = value;
                OnPropertyChanged(new PropertyChangedEventArgs("PrintPhotoCommand"));
            }
        }

        /// <summary>
        /// Gets or sets the command to delete the photo
        /// </summary>
        public ICommand DeletePhotoCommand
        {
            get
            {
                return _deletePhotoCommand;
            }
            set
            {
                _deletePhotoCommand = value;
                OnPropertyChanged(new PropertyChangedEventArgs("DeletePhotoCommand"));
            }
        }

        /// <summary>
        /// Gets or sets the command to save changes made to the photo
        /// </summary>
        public ICommand SaveChangesCommand
        {
            get
            {
                return _saveChangesCommand;
            }
            set
            {
                _saveChangesCommand = value;
                OnPropertyChanged(new PropertyChangedEventArgs("SaveChangesCommand"));
            }
        }

        /// <summary>
        /// Gets or sets the command to show the next photo in the album
        /// </summary>
        public ICommand ShowNextPhotoCommand
        {
            get
            {
                return _showNextPhotoCommand;
            }
            set
            {
                _showNextPhotoCommand = value;
                OnPropertyChanged(new PropertyChangedEventArgs("ShowNextPhotoCommand"));
            }
        }

        /// <summary>
        /// Gets or sets the command to show the previous photo in the album
        /// </summary>
        public ICommand ShowPreviousPhotoCommand
        {
            get
            {
                return _showPreviousPhotoCommand;
            }
            set
            {
                _showPreviousPhotoCommand = value;
                OnPropertyChanged(new PropertyChangedEventArgs("ShowPreviousPhotoCommand"));
            }
        }

        #endregion

        #region Public properties

        public Photo Original
        {
            get
            {
                return _original;
            }
        }

        /// <summary>
        /// Gets the date the photo was created
        /// </summary>
        public DateTime DateCreated
        {
            get
            {
                return _original.DateCreated;
            }
        }

        /// <summary>
        /// Gets or sets the title of the photo
        /// </summary>
        public string Title
        {
            get
            {
                return _original.Title;
            }
            set
            {
                _original.Title = value;
                OnPropertyChanged(new PropertyChangedEventArgs("Title"));
            }
        }

        /// <summary>
        /// Gets the thumbnail of the image
        /// </summary>
        public ImageSource Thumbnail
        {
            get
            {
                if (_retrievingThumbnail)
                    return null;

                _thumbnail = ImageCache.GetCachedImageSource(_original.Path + "_thumbnail");

                if (_thumbnail == null)
                {
                    _retrievingThumbnail = true;

                    // Load the thumbnail in a background thread
                    Dispatcher.CurrentDispatcher.BeginInvoke(new Action(() =>
                    {
                        BitmapImage img = new BitmapImage();

                        img.BeginInit();

                        img.StreamSource = _original.Load();
                        img.DecodePixelWidth = 360;
                        img.CacheOption = BitmapCacheOption.OnLoad;

                        img.EndInit();
                        img.Freeze();

                        _thumbnail = img;
                        _retrievingThumbnail = false;

                        ImageCache.SaveImageSourceInCache(_original.Path + "_thumbnail", img);

                        // Notify the UI that the thumbnail has been updated
                        OnPropertyChanged(new PropertyChangedEventArgs("Thumbnail"));
                    }), DispatcherPriority.Background);

                    // Return an empty thumbnail as placeholder
                    return null;
                }

                return _thumbnail;
            }
        }

        /// <summary>
        /// Gets or sets the zoom factor of the photo
        /// </summary>
        public double ZoomFactor
        {
            get
            {
                return _zoomFactor;
            }
            set
            {
                _zoomFactor = value;
                OnPropertyChanged(new PropertyChangedEventArgs("ZoomFactor"));
            }
        }

        /// <summary>
        /// Gets or sets the tags attached to the picture
        /// </summary>
        public Collection<string> Tags
        {
            get { return _original.Tags; }
            set
            {
                if (value != null)
                {
                    _original.Tags = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("Tags"));
                }
                else
                {
                    _original.Tags = new Collection<string>();
                    OnPropertyChanged(new PropertyChangedEventArgs("Tags"));
                }
            }
        }

        /// <summary>
        /// Gets the full image
        /// </summary>
        public ImageSource FullImage
        {
            get
            {
                _fullImage = ImageCache.GetCachedImageSource(_original.Path + "_full");

                if (_retrievingPhoto)
                    return null;

                if (_fullImage == null)
                {
                    _retrievingPhoto = true;

                    // Load the image in a background thread
                    Dispatcher.CurrentDispatcher.BeginInvoke(new Action(() =>
                    {
                        BitmapImage img = new BitmapImage();

                        img.BeginInit();

                        img.StreamSource = _original.Load();
                        img.CacheOption = BitmapCacheOption.OnLoad;

                        img.EndInit();
                        img.Freeze();

                        _fullImage = img;
                        _retrievingPhoto = false;

                        ImageCache.SaveImageSourceInCache(_original.Path + "_full", img);

                        // Notify the UI that the thumbnail has been updated
                        OnPropertyChanged(new PropertyChangedEventArgs("FullImage"));
                    }), DispatcherPriority.Background);

                    // Return an empty image as placeholder
                    return null;
                }

                return _fullImage;
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #region OnPropertyChanged
        /// <summary>
        /// Triggers the PropertyChanged event.
        /// </summary>
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs ea)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, ea);
        }
        #endregion

        #endregion
    }
}
