﻿using System;
using System.Windows;
using Microsoft.Practices.Composite.Wpf.Commands;
using Photoviewer.Common.Contract;
using Microsoft.Practices.Unity;
using Photoviewer.Common;

namespace Photoviewer.Modules.Photos.Views
{
    /// <summary>
    /// Presents photo details to the user
    /// </summary>
    public class PhotoDetailsPresenter
    {
        #region Private fields

        private readonly IPhotoDetailsView _view;
        private PhotoViewModel _currentPhoto;
        private IPrintService _printService;

        #endregion
        
        #region Public events
        
        public event EventHandler PhotoRemovalRequested;

        #endregion

        #region Public constructors

        /// <summary>
        /// Initializes a new instance of the PhotoDetailsPresenter class.
        /// </summary>
        /// <param name="view"></param>
        public PhotoDetailsPresenter(IPhotoDetailsView view)
        {
            _view = view;
        }

        #endregion

        #region Public methods

        /// <summary>
        /// Displays the details of the specified photo
        /// </summary>
        /// <param name="photo"></param>
        public void ShowPhotoDetails(PhotoViewModel photo)
        {
            _currentPhoto = photo;

            _currentPhoto.ShowNextPhotoCommand = new DelegateCommand<object>(OnNextPhotoRequested);
            _currentPhoto.ShowPreviousPhotoCommand = new DelegateCommand<object>(OnPreviousPhotoRequested);
            _currentPhoto.SaveChangesCommand = new DelegateCommand<object>(OnSaveChanges);
            _currentPhoto.DeletePhotoCommand = new DelegateCommand<object>(OnDeletePhoto);
            _currentPhoto.PrintPhotoCommand = new DelegateCommand<object>(OnPrintPhoto);

            _view.Model = _currentPhoto;

            DialogWorkspace.ShowView(_view, new DialogOptions
            {
                HideOkCancelButtons = true,
                AllowResize = true,
                DialogTitle = "Photo details",
                DialogSize = new Size { Width = 800, Height = 600 }
            }, null);
        }

        #endregion

        #region Command handlers

        /// <summary>
        /// Deletes the current photo
        /// </summary>
        /// <param name="args"></param>
        private void OnDeletePhoto(object args)
        {
            if (PhotoRemovalRequested != null)
                PhotoRemovalRequested(this, EventArgs.Empty);
        }

        /// <summary>
        /// Saves the changes to the photo
        /// </summary>
        /// <param name="args"></param>
        private void OnSaveChanges(object args)
        {
            // Force an update (sadly, tags don't have the nice property changed construction)
            _currentPhoto.Original.Save(true);
        }

        /// <summary>
        /// Prints the current photo
        /// </summary>
        /// <param name="args"></param>
        private void OnPrintPhoto(object args)
        {
            if(_printService == null)
            {
                MessageBox.Show(
                    "The print service is not available at the moment. Please check if the printing module is installed",
                    "Printing service is not initialized", MessageBoxButton.OK, MessageBoxImage.Error);

                return;
            }

            _printService.PrintPhoto(this._currentPhoto.Original);
        }

        /// <summary>
        /// Navigates to the next photo
        /// </summary>
        /// <param name="args"></param>
        private void OnNextPhotoRequested(object args)
        {
            int photoIndex = _currentPhoto.Original.Album.Photos.IndexOf(_currentPhoto.Original);

            if (photoIndex + 1 < _currentPhoto.Original.Album.Photos.Count)
            {
                _currentPhoto = new PhotoViewModel(_currentPhoto.Original.Album.Photos[photoIndex + 1]);

                // Bind the required commands to the view model
                _currentPhoto.ShowNextPhotoCommand = new DelegateCommand<object>(OnNextPhotoRequested);
                _currentPhoto.ShowPreviousPhotoCommand = new DelegateCommand<object>(OnPreviousPhotoRequested);

                _view.Model = _currentPhoto;
            }
        }

        /// <summary>
        /// Navigates to the previous photo
        /// </summary>
        /// <param name="args"></param>
        private void OnPreviousPhotoRequested(object args)
        {
            int photoIndex = _currentPhoto.Original.Album.Photos.IndexOf(_currentPhoto.Original);

            if (photoIndex - 1 >= 0)
            {
                _currentPhoto = new PhotoViewModel(_currentPhoto.Original.Album.Photos[photoIndex - 1]);

                // Bind the required commands to the view model
                _currentPhoto.ShowNextPhotoCommand = new DelegateCommand<object>(OnNextPhotoRequested);
                _currentPhoto.ShowPreviousPhotoCommand = new DelegateCommand<object>(OnPreviousPhotoRequested);

                _view.Model = _currentPhoto;
            }
        }
        
        #endregion

        #region Public properties

        /// <summary>
        /// Gets the photo that is currently displayed
        /// </summary>
        public PhotoViewModel CurrentPhoto
        {
            get
            {
                return _currentPhoto;
            }
        }

        [Dependency]
        public IPrintService PrintService
        {
            get { return _printService; }
            set { _printService = value; }
        }
        
        #endregion
    }
}
