﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using Photoviewer.DomainModel;
using System.Windows.Controls;
using System.Windows.Markup;

namespace Photoviewer.Modules.Printing
{
    /// <summary>
    /// Contains all the logic required to create a document suitable to print the 
    /// contents of a whole album
    /// </summary>
    internal class AlbumDocumentBuilder
    {
        private FixedDocument _document;
        private FixedPage _currentPage;
        private FixedPage _lastPage;
        private UniformGrid _photoGrid;
        private Album _album;

        public double PageWidth { get; set; }
        public double PageHeight { get; set; }

        /// <summary>
        /// Builds a document sequence that can be printed
        /// </summary>
        /// <param name="album"></param>
        /// <returns></returns>
        public FixedDocumentSequence BuildDocumentSequence(Album album)
        {
            _album = album;

            CreateDocument();
            
            FixedDocumentSequence sequence = new FixedDocumentSequence();
            DocumentReference reference = new DocumentReference();

            reference.SetDocument(_document);
            sequence.References.Add(reference);

            return sequence;
        }

        /// <summary>
        /// Creates the document that needs to be printed
        /// </summary>
        private void CreateDocument()
        {
            _document = new FixedDocument();

            CreatePage();

            for(var index = 0; index < _album.Photos.Count; index++)
            {
                if(index % 6 == 0 && index > 0)
                {
                    SaveCurrentPage();
                    CreatePage();
                }

                CreatePicture(_album.Photos[index]);
            }

            if(_lastPage != _currentPage)
            {
                SaveCurrentPage();
            }
        }

        /// <summary>
        /// Saves the current page
        /// </summary>
        private void SaveCurrentPage()
        {
            PageContent content = new PageContent();
            IAddChild childContainer = (IAddChild) content;

            _photoGrid.Width = PageWidth - 192;
            _photoGrid.Height = PageHeight - 192;

            _photoGrid.Measure(new Size(PageWidth - 192,PageHeight - 192));
            _photoGrid.Arrange(new Rect(0,0,PageWidth - 192,PageHeight - 192));

            childContainer.AddChild(_currentPage);
            _document.Pages.Add(content);

            _lastPage = _currentPage;
        }

        /// <summary>
        /// Creates a new page
        /// </summary>
        private void CreatePage()
        {
            _currentPage = new FixedPage();
            _photoGrid = new UniformGrid();

            _photoGrid.Columns = 2;
            _photoGrid.Margin = new Thickness(86);

            _currentPage.Children.Add(_photoGrid);
        }

        /// <summary>
        /// Creates a printable version of a photo
        /// </summary>
        /// <param name="original"></param>
        private void CreatePicture(Photo original)
        {
            PrintablePhoto photo = new PrintablePhoto(original);
            Image img = new Image();

            img.Source = photo.ImageData;
            img.Stretch = Stretch.Uniform;
            img.Margin = new Thickness(5);

            _photoGrid.Children.Add(img);
        }
    }
}
