﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Photoviewer.DataAccess.Contract;
using Photoviewer.DataAccess.Implementation.Metadata;

namespace Photoviewer.DataAccess.Implementation.Fixtures
{
    /// <summary>
    /// Fixture to check the album data provider
    /// </summary>
    [TestClass]
    public class AlbumDataProviderFixture
    {
        /// <summary>
        /// Tries to retrieve albums from a non-existing base path
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void GetAlbumsFromNonExistingBasePath()
        {
            AlbumDataProvider provider = new AlbumDataProvider("C:\\Gibberish123532546");
            provider.GetAlbums();
        }

        /// <summary>
        /// Gets a set of albums without metadata files available
        /// </summary>
        [TestMethod]
        public void GetAlbumsWithoutExistingMetadata()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);
            
            Directory.CreateDirectory(basePath);
            Directory.CreateDirectory(Path.Combine(basePath, "Album1"));
            Directory.CreateDirectory(Path.Combine(basePath, "Album2"));

            try
            {
                Collection<AlbumData> data = provider.GetAlbums();

                Assert.AreEqual(2, data.Count);
                Assert.AreEqual("Album1", data[0].Title);
            }
            finally
            {
                Directory.Delete(Path.Combine(basePath, "Album1"));
                Directory.Delete(Path.Combine(basePath, "Album2"));
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Gets all available albums with metadata defined
        /// </summary>
        [TestMethod]
        public void GetAlbumsWithMetadata()
        {
            string basePath = Path.GetFullPath("Albums");

            Func<string,AlbumMetadata> CreateMetadata = (title) => {
                return new AlbumMetadata { Title = title };
            };

            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);
            Directory.CreateDirectory(Path.Combine(basePath, "Album1"));
            Directory.CreateDirectory(Path.Combine(basePath, "Album2"));

            AlbumMetadata album1Metadata = CreateMetadata("Test album 1");
            AlbumMetadata album2Metadata = CreateMetadata("Test album 2");

            album1Metadata.Save(Path.Combine(basePath, "Album1"));
            album2Metadata.Save(Path.Combine(basePath, "Album2"));

            try
            {
                Collection<AlbumData> data = provider.GetAlbums();

                Assert.AreEqual(2, data.Count);
                Assert.AreEqual("Test album 1", data[0].Title);
                Assert.AreEqual("Test album 2", data[1].Title);
            }
            finally
            {
                File.Delete(Path.Combine(Path.Combine(basePath,"Album1"),".metadata.xml"));
                File.Delete(Path.Combine(Path.Combine(basePath,"Album2"),".metadata.xml"));
                Directory.Delete(Path.Combine(basePath, "Album1"));
                Directory.Delete(Path.Combine(basePath, "Album2"));
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Retrieves a single album with metadata
        /// </summary>
        [TestMethod]
        public void GetExistingAlbumWithMetadata()
        {
            string basePath = Path.GetFullPath("Albums");

            Func<string, AlbumMetadata> CreateMetadata = (title) =>
            {
                return new AlbumMetadata { Title = title };
            };

            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);
            Directory.CreateDirectory(Path.Combine(basePath, "Album1"));

            AlbumMetadata album1Metadata = CreateMetadata("Test album 1");

            album1Metadata.Save(Path.Combine(basePath, "Album1"));

            try
            {
                string albumPath = "Album1";
                AlbumData data = provider.GetAlbumByPath(albumPath);

                Assert.AreEqual("Test album 1", data.Title);
                Assert.AreEqual(Path.Combine(basePath,"Album1"),data.Path);
            }
            finally
            {
                File.Delete(Path.Combine(Path.Combine(basePath, "Album1"), ".metadata.xml"));
                Directory.Delete(Path.Combine(basePath, "Album1"));
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Retrieves a single album without metadata
        /// </summary>
        [TestMethod]
        public void GetAlbumWithoutMetadata()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);
            Directory.CreateDirectory(Path.Combine(basePath, "Album1"));

            try
            {
                string albumPath = "Album1";
                AlbumData data = provider.GetAlbumByPath(albumPath);

                Assert.AreEqual("Album1", data.Title);
                Assert.AreEqual(Path.Combine(basePath, "Album1"), data.Path);
            }
            finally
            {
                Directory.Delete(Path.Combine(basePath, "Album1"));
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Tries to retrieve metadata of a non-existing album
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void GetNonExistingAlbum()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);

            try
            {
                string albumPath = "Album1";
                provider.GetAlbumByPath(albumPath);
            }
            finally
            {
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Tries to retrieve album metadata with an invalid base path
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void GetAlbumWithInvalidBasePath()
        {
            string invalidPath = "C:\\gibberish123152356345";
            AlbumDataProvider provider = new AlbumDataProvider(invalidPath);

            provider.GetAlbumByPath("Album1");
        }

        /// <summary>
        /// Creates a new album
        /// </summary>
        [TestMethod]
        public void CreateAlbumWithRelativePath()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);

            try
            {
                provider.CreateAlbum(new AlbumData
                                         {
                    Title = "Test album 1",
                    Path = "Album1"
                });

                AlbumData data = provider.GetAlbumByPath("Album1");

                Assert.AreEqual("Test album 1", data.Title);
            }
            finally
            {
                File.Delete(Path.Combine(basePath, Path.Combine("Album1", ".metadata.xml")));
                Directory.Delete(Path.Combine(basePath, "Album1"));
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Creates an album with an absolute path
        /// </summary>
        [TestMethod]
        public void CreateAlbumWithAbsolutePath()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);

            try
            {
                provider.CreateAlbum(new AlbumData
                                         {
                    Title = "Test album 1",
                    Path = Path.Combine(basePath, "Album1")
                });

                AlbumData data = provider.GetAlbumByPath("Album1");

                Assert.AreEqual("Test album 1", data.Title);
            }
            finally
            {
                try
                {
                    File.Delete(Path.Combine(basePath, Path.Combine("Album1", ".metadata.xml")));
                    Directory.Delete(Path.Combine(basePath, "Album1"));
                    Directory.Delete(basePath);
                }
                catch
                {

                }
            }
        }

        /// <summary>
        /// Creates an album with an invalid base path
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void CreateAlbumWithInvalidPath()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);

            try
            {
                provider.CreateAlbum(new AlbumData
                                         {
                    Title = "Test album 1",
                    Path = Path.Combine("C:\\test", "Album1")
                });

                provider.GetAlbumByPath("Album1");
            }
            finally
            {
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Updates the metadata of an existing album
        /// </summary>
        [TestMethod]
        public void UpdateAlbum()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            Directory.CreateDirectory(basePath);

            try
            {
                provider.CreateAlbum(new AlbumData
                                         {
                    Title = "Test album 1",
                    Path = "Album1"
                });

                AlbumData data = provider.GetAlbumByPath("Album1");
                data.Title = "Album test 2";

                provider.UpdateAlbum(data);
                data = provider.GetAlbumByPath("Album1");

                Assert.AreEqual("Album test 2", data.Title);
            }
            finally
            {
                File.Delete(Path.Combine(basePath, Path.Combine("Album1", ".metadata.xml")));
                Directory.Delete(Path.Combine(basePath, "Album1"));
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Tries to update a non-existing album
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void UpdateNonExistingAlbum()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            try
            {
                Directory.CreateDirectory(basePath);

                AlbumData data = new AlbumData
                {
                    Title = "Test title",
                    Path = Path.Combine(basePath, "Album1")
                };

                provider.UpdateAlbum(data);
            }
            finally
            {
                Directory.Delete(basePath);   
            }
        }

        /// <summary>
        /// Deletes an existing album
        /// </summary>
        [TestMethod]
        public void DeleteExistingAlbumWithoutFiles()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            try
            {
                Directory.CreateDirectory(basePath);
                Directory.CreateDirectory(Path.Combine(basePath, "Album1"));

                provider.DeleteAlbum(Path.Combine(basePath, "Album1"));

                Assert.IsFalse(Directory.Exists(Path.Combine(basePath, "Album1")));
            }
            finally
            {
                Directory.Delete(basePath);
            }
        }

        /// <summary>
        /// Deletes an existing album
        /// </summary>
        [TestMethod]
        public void DeleteExistingAlbumWithFiles()
        {
            string basePath = Path.GetFullPath("Albums");
            AlbumDataProvider provider = new AlbumDataProvider(basePath);

            try
            {
                Directory.CreateDirectory(basePath);
                Directory.CreateDirectory(Path.Combine(basePath, "Album1"));

                AlbumMetadata metadata = new AlbumMetadata();
                metadata.Title = "Test";
                metadata.Save(Path.Combine(basePath, "Album1"));

                provider.DeleteAlbum(Path.Combine(basePath, "Album1"));

                Assert.IsFalse(Directory.Exists(Path.Combine(basePath, "Album1")));
            }
            finally
            {
                Directory.Delete(basePath);
            }
        }
    }
}
