﻿using System.Collections.ObjectModel;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Photoviewer.DataAccess.Implementation.Metadata;

namespace Photoviewer.DataAccess.Implementation.Fixtures
{
    /// <summary>
    /// Fixture to check the business logic of the album metadata class
    /// </summary>
    [TestClass]
    public class AlbumMetadataFixture
    {
        private const string AlbumDirectory = "TestAlbum";
        private const string NewAlbumDirectory = "TestAlbum2";

        /// <summary>
        /// Initializes the fixture
        /// </summary>
        [TestInitialize]
        public void InitializeTest()
        {
            Directory.CreateDirectory(Path.GetFullPath(NewAlbumDirectory));
            Directory.CreateDirectory(Path.GetFullPath(AlbumDirectory));
        }

        /// <summary>
        /// Cleans up used resources
        /// </summary>
        [TestCleanup]
        public void TearDownTest()
        {
            string metadataPath = Path.Combine(Path.GetFullPath(AlbumDirectory), ".metadata.xml");
            
            if (File.Exists(metadataPath))
            {
                File.Delete(metadataPath);
            }

            Directory.Delete(Path.GetFullPath(AlbumDirectory));
            Directory.Delete(Path.GetFullPath(NewAlbumDirectory));
        }

        /// <summary>
        /// Saves metadata
        /// </summary>
        [TestMethod]
        public void SaveMetadata()
        {
            AlbumMetadata metadata = new AlbumMetadata();
            metadata.Title = "Test";
            metadata.Photos.Add(new Photo
                                    { 
                Title = "Test", 
                Path = Path.Combine(Path.GetFullPath(AlbumDirectory), "test.jpg"), 
                Tags = new Collection<string> { "Test 1","Test 2" } 
            });

            metadata.Save(Path.GetFullPath(AlbumDirectory));

            Assert.IsTrue(File.Exists(Path.Combine(Path.GetFullPath(AlbumDirectory), ".metadata.xml")));
        }

        /// <summary>
        /// Loads metadata
        /// </summary>
        [TestMethod]
        public void LoadMetadata()
        {
            AlbumMetadata metadata = new AlbumMetadata();
            metadata.Title = "Test";
            metadata.Photos.Add(new Photo
                                    {
                Title = "Test",
                Path = Path.Combine(Path.GetFullPath(AlbumDirectory), "test.jpg"),
                Tags = new Collection<string> { "Test 1", "Test 2" }
            });

            metadata.Save(Path.GetFullPath(AlbumDirectory));

            AlbumMetadata loadedData = AlbumMetadata.Load(Path.GetFullPath(AlbumDirectory));

            Assert.AreEqual(1, loadedData.Photos.Count);
            Assert.AreEqual(metadata.Title, loadedData.Title);
            Assert.AreEqual(2, metadata.Photos[0].Tags.Count);
        }

        /// <summary>
        /// Tries to load metadata of an album that doesnt have metadata yet
        /// </summary>
        [TestMethod]
        public void LoadNewMetadata()
        {
            AlbumMetadata metadata = AlbumMetadata.Load("C:\\test");

            Assert.AreEqual(0, metadata.Photos.Count);
            Assert.IsNull(metadata.Title);
        }
    }
}
