﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Photoviewer.DataAccess.Contract;
using Photoviewer.DataAccess.Implementation.Metadata;

namespace Photoviewer.DataAccess.Implementation.Fixtures
{
    /// <summary>
    /// Fixture to check the functionality of the photo data provider
    /// </summary>
    [TestClass]
    public class PhotoDataProviderFixture
    {
        /// <summary>
        /// Creates new photo metadata
        /// </summary>
        [TestMethod]
        public void CreatePhotoMetadata()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                using (FileStream stream = File.Create(Path.GetFullPath("Albums\\Album1\\test.jpg")))
                {
                    stream.Write(Encoding.ASCII.GetBytes("Test"),0,4);
                }

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.CreatePhotoMetadata(new PhotoData
                                                 {
                    Path = "test.jpg",
                    AlbumPath = Path.GetFullPath("Albums\\Album1"),
                    Title = "Test"
                });
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Tries to create photo metadata without an existing photo
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(PhotoNotFoundException))]
        public void CreatePhotoMetadataWithoutExistingPhoto()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.CreatePhotoMetadata(new PhotoData
                                                 {
                    Path = "test.jpg",
                    AlbumPath = Path.GetFullPath("Albums\\Album1"),
                    Title = "Test"
                });
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Tries to create photo metadata without an existing photo
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void CreatePhotoMetadataForExistingPhoto()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";
                data.Photos.Add(new Photo { Title = "Test", Path = Path.GetFullPath("Albums\\Album1\\test.jpg") });

                data.Save(Path.GetFullPath("Albums\\Album1"));

                using (FileStream stream = File.Create(Path.GetFullPath("Albums\\Album1\\test.jpg")))
                {
                    stream.Write(Encoding.ASCII.GetBytes("Test"), 0, 4);
                }

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.CreatePhotoMetadata(new PhotoData
                                                 {
                    Path = "test.jpg",
                    AlbumPath = Path.GetFullPath("Albums\\Album1"),
                    Title = "Test"
                });
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Gets an existing photo
        /// </summary>
        [TestMethod]
        public void GetExistingPhotoByPath()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                using (FileStream stream = File.Create(Path.GetFullPath("Albums\\Album1\\test.jpg")))
                {
                    stream.Write(Encoding.ASCII.GetBytes("Test"), 0, 4);
                }

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.CreatePhotoMetadata(new PhotoData
                                                 {
                    Path = "test.jpg",
                    AlbumPath = Path.GetFullPath("Albums\\Album1"),
                    Title = "Test"
                });

                PhotoData returnedData = provider.GetPhotoByPath(Path.GetFullPath("Albums\\Album1\\test.jpg"));

                Assert.IsNotNull(returnedData);
                Assert.AreEqual("Test", returnedData.Title);
                Assert.AreEqual(0, returnedData.Tags.Count);
                Assert.AreEqual(Path.GetFullPath("Albums\\Album1"), returnedData.AlbumPath);
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Retrieves a photo from disk without existing metadata
        /// </summary>
        [TestMethod]
        public void GetPhotoByPathWithoutExistingMetadata()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                using (FileStream stream = File.Create(Path.GetFullPath("Albums\\Album1\\test.jpg")))
                {
                    stream.Write(Encoding.ASCII.GetBytes("Test"), 0, 4);
                }

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                
                PhotoData returnedData = provider.GetPhotoByPath(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                FileInfo photoInfo = new FileInfo(Path.GetFullPath("Albums\\Album1\\test.jpg"));

                Assert.IsNotNull(returnedData);
                Assert.AreEqual(0,returnedData.Tags.Count);
                Assert.AreEqual(photoInfo.Name, returnedData.Title);
                Assert.AreEqual(photoInfo.Directory.FullName, returnedData.AlbumPath);
                Assert.AreEqual(photoInfo.FullName, returnedData.Path);
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Gets a non-existing photo
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GetNonExistingPhotoByPath()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.GetPhotoByPath(Path.GetFullPath("Albums\\Album1\\test2.jpg"));
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Updates the metadata of an existing photo
        /// </summary>
        [TestMethod]
        public void UpdatePhotoMetadata()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                using (FileStream stream = File.Create(Path.GetFullPath("Albums\\Album1\\test.jpg")))
                {
                    stream.Write(Encoding.ASCII.GetBytes("Test"), 0, 4);
                }

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.CreatePhotoMetadata(new PhotoData
                                                 {
                    Path = "test.jpg",
                    AlbumPath = Path.GetFullPath("Albums\\Album1"),
                    Title = "Test"
                });

                PhotoData existingPhoto = new PhotoData
                                              {
                    Title = "Test 123",
                    Path= Path.GetFullPath("Albums\\Album1\\test.jpg"),
                    AlbumPath = Path.GetFullPath("Albums\\Album1")
                };

                provider.UpdatePhotoMetadata(existingPhoto);

                AlbumMetadata savedData = AlbumMetadata.Load(Path.GetFullPath("Albums\\Album1"));
                var foundData = savedData.Photos.Where(p => p.Path == Path.GetFullPath("Albums\\Album1\\test.jpg"));

                Assert.AreNotEqual(0, foundData.Count());
                Assert.AreEqual("Test 123", foundData.First().Title);
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Tries to update a non-existing photo
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(PhotoNotFoundException))]
        public void UpdateNonExistingPhoto()
        {
            var provider = new PhotoDataProvider("C:\\Gibberish123532546");
            provider.UpdatePhotoMetadata(new PhotoData { Path = "C:\\girbberwere.jpg" });
        }

        /// <summary>
        /// Deletes an existing photo
        /// </summary>
        [TestMethod]
        public void DeleteExistingPhoto()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                using (FileStream stream = File.Create(Path.GetFullPath("Albums\\Album1\\test.jpg")))
                {
                    stream.Write(Encoding.ASCII.GetBytes("Test"), 0, 4);
                }

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.CreatePhotoMetadata(new PhotoData
                                                 {
                    Path = "test.jpg",
                    AlbumPath = Path.GetFullPath("Albums\\Album1"),
                    Title = "Test"
                });

                provider.DeletePhoto(Path.GetFullPath("Albums\\Album1\\test.jpg"));

                Assert.IsFalse(File.Exists(Path.GetFullPath("Albums\\Album1\\test.jpg")));
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }

        /// <summary>
        /// Deletes an existing photo
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(PhotoNotFoundException))]
        public void DeleteNonExistingPhoto()
        {
            var provider = new PhotoDataProvider("C:\\Gibberish123532546");
            provider.DeletePhoto("C:\\testwgoiewrjgerg.jpg");
        }

        /// <summary>
        /// Retrieves all photos from a single album
        /// </summary>
        [TestMethod]
        public void GetPhotosByAlbum()
        {
            try
            {
                Directory.CreateDirectory("Albums");
                Directory.CreateDirectory("Albums\\Album1");

                AlbumMetadata data = new AlbumMetadata();
                data.Title = "Test";

                data.Save(Path.GetFullPath("Albums\\Album1"));

                using (FileStream stream = File.Create(Path.GetFullPath("Albums\\Album1\\test.jpg")))
                {
                    stream.Write(Encoding.ASCII.GetBytes("Test"), 0, 4);
                }

                PhotoDataProvider provider = new PhotoDataProvider("C:\\Gibberish123532546");
                provider.CreatePhotoMetadata(new PhotoData
                                                 {
                    Path = "test.jpg",
                    AlbumPath = Path.GetFullPath("Albums\\Album1"),
                    Title = "Test"
                });

                Collection<PhotoData> result = provider.GetPhotosByAlbumPath(
                    Path.GetFullPath("Albums\\Album1"));

                Assert.AreNotEqual(0, result);
                Assert.AreEqual("Test", result[0].Title);
            }
            finally
            {
                File.Delete(Path.GetFullPath("Albums\\Album1\\test.jpg"));
                File.Delete(Path.GetFullPath("Albums\\Album1\\.metadata.xml"));
                Directory.Delete("Albums\\Album1");
                Directory.Delete("Albums");
            }
        }
    }
}
