﻿using System;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Photoviewer.DataAccess.Contract;
using Rhino.Mocks;

namespace Photoviewer.DomainModel.Fixtures
{
    /// <summary>
    /// Test fixture for the album domain model class
    /// </summary>
    [TestClass]
    public class AlbumFixture
    {
        /// <summary>
        /// Creates a new album
        /// </summary>
        [TestMethod]
        public void TestCreateAlbum()
        {
            Album album = Album.NewAlbum("C:\\test");

            Assert.AreEqual("C:\\test", album.Path);
            Assert.AreEqual(String.Empty,album.Title);
        }

        /// <summary>
        /// Creates a new album with an empty path
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCreateAlbumEmptyPath()
        {
            Album newAlbum = Album.NewAlbum(null);
        }

        /// <summary>
        /// Retrieves an album by providing raw data
        /// </summary>
        [TestMethod]
        public void GetAlbumFromRawData()
        {
            var data = new AlbumData
                           {
                Path = "C:\\test",
                Title = "test"
            };

            Album album = Album.GetAlbum(data);

            Assert.AreEqual(data.Title, album.Title);
            Assert.AreEqual(data.Path, album.Path);

            Assert.IsFalse(album.IsNew);
            Assert.IsFalse(album.IsDirty);
        }

        /// <summary>
        /// Creates a new album and saves it
        /// </summary>
        [TestMethod]
        public void TestSaveNewAlbum()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider provider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "test"
            };

            using (repository.Record())
            {
                provider = repository.StrictMock<IAlbumDataProvider>();
                Expect.Call(provider.CreateAlbum(savedData)).Return(savedData);
            }

            container.RegisterInstance(provider);
            ModelContext.Initialize(container);

            Album album = Album.NewAlbum("C:\\test");
            album.Title = "test";

            album = album.Save();

            Assert.IsFalse(album.IsNew);
            Assert.IsFalse(album.IsDeleted);

            repository.VerifyAll();
        }

        /// <summary>
        /// Retrieves an existing album
        /// </summary>
        [TestMethod]
        public void GetExistingAlbum()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider provider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData originalData = new AlbumData
                                         {
                Path = "C:\\test",
                Title = "test"
            };

            using (repository.Record())
            {
                provider = repository.StrictMock<IAlbumDataProvider>();

                Expect.Call(provider.GetAlbumByPath("C:\\test")).Return(originalData);
            }

            container.RegisterInstance(provider);
            ModelContext.Initialize(container);

            Album album = Album.GetAlbum("C:\\test");

            Assert.IsFalse(album.IsNew);
            Assert.IsFalse(album.IsDirty);
            Assert.AreEqual(originalData.Title, album.Title);
            Assert.AreEqual(originalData.Path, album.Path);

            repository.VerifyAll();
        }

        /// <summary>
        /// Retrieves an existing album and updates it
        /// </summary>
        [TestMethod]
        public void TestUpdateAlbum()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider provider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData originalData = new AlbumData
                                         {
                Path = "C:\\test",
                Title = "test"
            };

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "Test 2"
            };

            using (repository.Record())
            {
                provider = repository.StrictMock<IAlbumDataProvider>();

                Expect.Call(provider.GetAlbumByPath("C:\\test")).Return(originalData);
                Expect.Call(provider.UpdateAlbum(savedData)).Return(savedData);
            }

            container.RegisterInstance(provider);
            ModelContext.Initialize(container);

            Album album = Album.GetAlbum("C:\\test");
            album.Title = "Test 2";
            album = album.Save();

            Assert.IsFalse(album.IsNew);
            Assert.IsFalse(album.IsDirty);

            repository.VerifyAll();
        }

        /// <summary>
        /// Retrieves an existing album
        /// </summary>
        [TestMethod]
        public void TestDeleteExistingAlbum()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider provider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData originalData = new AlbumData
                                         {
                Path = "C:\\test",
                Title = "test"
            };

            using (repository.Record())
            {
                provider = repository.StrictMock<IAlbumDataProvider>();

                Expect.Call(provider.GetAlbumByPath("C:\\test")).Return(originalData);
                provider.DeleteAlbum("C:\\test");
            }

            container.RegisterInstance(provider);
            ModelContext.Initialize(container);

            Album album = Album.GetAlbum("C:\\test");
            album.Delete();
            album.Save();

            Assert.IsTrue(album.IsDeleted);

            repository.VerifyAll();
        }

        /// <summary>
        /// Deletes an album without retrieving it first
        /// </summary>
        [TestMethod]
        public void DeleteAlbumDirect()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider provider = null;
            IUnityContainer container = new UnityContainer();

            using (repository.Record())
            {
                provider = repository.StrictMock<IAlbumDataProvider>();
                provider.DeleteAlbum("C:\\test");
            }

            container.RegisterInstance(provider);
            ModelContext.Initialize(container);

            Album.DeleteAlbum("C:\\test");

            repository.VerifyAll();
        }

        /// <summary>
        /// Tries to delete an album without providing a valid path
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DeleteAlbumDirectWithoutPath()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider provider = null;
            IUnityContainer container = new UnityContainer();

            using (repository.Record())
            {
                provider = repository.StrictMock<IAlbumDataProvider>();
                provider.DeleteAlbum("C:\\test");
            }

            container.RegisterInstance(provider);
            ModelContext.Initialize(container);

            Album.DeleteAlbum(null);
        }
    }
}
