﻿using System;
using System.Collections.ObjectModel;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Photoviewer.DataAccess.Contract;
using Rhino.Mocks;

namespace Photoviewer.DomainModel.Fixtures
{
    /// <summary>
    /// Fixture to test the business logic of the photo
    /// </summary>
    [TestClass]
    public class PhotoFixture
    {
        /// <summary>
        /// Creates a new photo
        /// </summary>
        [TestMethod]
        public void CreatePhoto()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider provider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "test"
            };

            using (repository.Record())
            {
                provider = repository.StrictMock<IAlbumDataProvider>();
                Expect.Call(provider.GetAlbumByPath("C:\\test")).Return(savedData);
            }

            container.RegisterInstance(provider);
            ModelContext.Initialize(container);

            Album album = Album.GetAlbum("C:\\test");
            Photo photo = Photo.NewPhoto(album, "C:\\test\\test.jpg");

            Assert.AreEqual(album, photo.Album);
            Assert.AreEqual("C:\\test\\test.jpg", photo.Path);
            Assert.AreEqual(String.Empty, photo.Title);
            Assert.IsNotNull(photo.Tags);
            Assert.AreEqual(0, photo.Tags.Count);

            repository.VerifyAll();
        }

        /// <summary>
        /// Retrieves a single photo
        /// </summary>
        [TestMethod]
        public void GetExistingPhoto()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider albumProvider = null;
            IPhotoDataProvider photoProvider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "test"
            };

            PhotoData photoData = new PhotoData
                                      {
                Title = "Test",
                Path = "C:\\test\\test.jpg",
                AlbumPath = "C:\\test",
                Tags = new Collection<string>
                           {
                    "Test 1",
                    "Test 2"
                }
            };

            using (repository.Record())
            {
                albumProvider = repository.StrictMock<IAlbumDataProvider>();
                photoProvider = repository.StrictMock<IPhotoDataProvider>();
                Expect.Call(albumProvider.GetAlbumByPath("C:\\test")).Return(savedData);
                Expect.Call(photoProvider.GetPhotoByPath("C:\\test\\test.jpg")).Return(photoData);
            }

            container.RegisterInstance(albumProvider);
            container.RegisterInstance(photoProvider);

            ModelContext.Initialize(container);

            Photo photo = Photo.GetPhoto("C:\\test\\test.jpg");

            Assert.AreEqual(photoData.Title, photo.Title);
            Assert.AreEqual(2, photo.Tags.Count);
            Assert.IsNotNull(photo.Album);
            Assert.AreEqual(savedData.Title, photo.Album.Title);

            repository.VerifyAll();
        }

        /// <summary>
        /// Gets an existing photo using the method for retrieving
        /// a photo as part of retrieving the contents of an album
        /// </summary>
        [TestMethod]
        public void TestGetExistingPhotoFromAlbum()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider albumProvider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "test"
            };

            PhotoData photoData = new PhotoData
                                      {
                Title = "Test",
                Path = "C:\\test\\test.jpg",
                AlbumPath = "C:\\test",
                Tags = new Collection<string>
                           {
                    "Test 1",
                    "Test 2"
                }
            };

            using (repository.Record())
            {
                albumProvider = repository.StrictMock<IAlbumDataProvider>();
                Expect.Call(albumProvider.GetAlbumByPath("C:\\test")).Return(savedData);
            }

            container.RegisterInstance(albumProvider);

            ModelContext.Initialize(container);

            Album album = Album.GetAlbum("C:\\test");
            Photo photo = Photo.GetPhoto(album,photoData);

            Assert.AreEqual(photoData.Title, photo.Title);
            Assert.AreEqual(2, photo.Tags.Count);
            Assert.IsNotNull(photo.Album);
            Assert.AreEqual(savedData.Title, photo.Album.Title);

            repository.VerifyAll();
        }
        
        /// <summary>
        /// Creates a new photo and saves the metadata
        /// </summary>
        [TestMethod]
        public void SaveNewPhoto()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider albumProvider = null;
            IPhotoDataProvider photoProvider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "test"
            };

            PhotoData photoData = new PhotoData
                                      {
                Title = "Test",
                Path = "C:\\test\\test.jpg",
                AlbumPath = "C:\\test",
                Tags = new Collection<string>
                           {
                    "Test 1",
                    "Test 2"
                }
            };

            using (repository.Record())
            {
                albumProvider = repository.StrictMock<IAlbumDataProvider>();
                photoProvider = repository.StrictMock<IPhotoDataProvider>();
                Expect.Call(albumProvider.GetAlbumByPath("C:\\test")).Return(savedData);
                Expect.Call(photoProvider.CreatePhotoMetadata(photoData)).Return(photoData);
            }

            container.RegisterInstance(albumProvider);
            container.RegisterInstance(photoProvider);

            ModelContext.Initialize(container);

            Album album = Album.GetAlbum("C:\\test");
            Photo newPhoto = Photo.NewPhoto(album, "C:\\test\\test.jpg");
            
            newPhoto.Title = "Test";
            newPhoto.Tags.Add("Test 1");
            newPhoto.Tags.Add("Test 2");

            newPhoto = newPhoto.Save();

            Assert.IsFalse(newPhoto.IsNew);
            Assert.IsFalse(newPhoto.IsDirty);

            repository.VerifyAll();
        }

        /// <summary>
        /// Saves an existing photo
        /// </summary>
        [TestMethod]
        public void SaveExistingPhoto()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider albumProvider = null;
            IPhotoDataProvider photoProvider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "test"
            };

            PhotoData originalPhotoData = new PhotoData
                                              {
                Title = "Test",
                Path = "C:\\test\\test.jpg",
                AlbumPath = "C:\\test",
                Tags = new Collection<string>
                           {
                    "Test 1",
                    "Test 2"
                }
            };

            PhotoData photoData = new PhotoData
                                      {
                Title = "Test 2",
                Path = "C:\\test\\test.jpg",
                AlbumPath = "C:\\test",
                Tags = new Collection<string>
                           {
                    "Test 1",
                    "Test 2"
                }
            };

            using (repository.Record())
            {
                albumProvider = repository.StrictMock<IAlbumDataProvider>();
                photoProvider = repository.StrictMock<IPhotoDataProvider>();
                Expect.Call(albumProvider.GetAlbumByPath("C:\\test")).Return(savedData);
                Expect.Call(photoProvider.GetPhotoByPath("C:\\test\\test.jpg")).Return(originalPhotoData);
                Expect.Call(photoProvider.UpdatePhotoMetadata(photoData)).Return(photoData);
            }

            container.RegisterInstance(albumProvider);
            container.RegisterInstance(photoProvider);

            ModelContext.Initialize(container);
            Photo originalPhoto = Photo.GetPhoto("C:\\test\\test.jpg");

            originalPhoto.Title = "Test 2";
            Photo savedPhoto = originalPhoto.Save();

            Assert.IsFalse(savedPhoto.IsDirty);

            repository.VerifyAll();
        }

        [TestMethod]
        public void DeleteExistingPhoto()
        {
            MockRepository repository = new MockRepository();
            IAlbumDataProvider albumProvider = null;
            IPhotoDataProvider photoProvider = null;
            IUnityContainer container = new UnityContainer();

            AlbumData savedData = new AlbumData
                                      {
                Path = "C:\\test",
                Title = "test"
            };

            PhotoData photoData = new PhotoData
                                      {
                Title = "Test",
                Path = "C:\\test\\test.jpg",
                AlbumPath = "C:\\test",
                Tags = new Collection<string>
                           {
                    "Test 1",
                    "Test 2"
                }
            };

            using (repository.Record())
            {
                albumProvider = repository.StrictMock<IAlbumDataProvider>();
                photoProvider = repository.StrictMock<IPhotoDataProvider>();
                
                Expect.Call(albumProvider.GetAlbumByPath("C:\\test")).Return(savedData);
                Expect.Call(photoProvider.GetPhotoByPath("C:\\test\\test.jpg")).Return(photoData);

                photoProvider.DeletePhoto("C:\\test\\test.jpg");
            }

            container.RegisterInstance(albumProvider);
            container.RegisterInstance(photoProvider);

            ModelContext.Initialize(container);

            Photo photo = Photo.GetPhoto("C:\\test\\test.jpg");
            photo.Delete();
            photo.Save();

            Assert.IsTrue(photo.IsDeleted);

            repository.VerifyAll();
        }

        /// <summary>
        /// Deletes a photo by using the path to the photo
        /// </summary>
        [TestMethod]
        public void DeletePhotoDirect()
        {
            MockRepository repository = new MockRepository();
            IPhotoDataProvider photoProvider = null;
            IUnityContainer container = new UnityContainer();

            using (repository.Record())
            {
                photoProvider = repository.StrictMock<IPhotoDataProvider>();
                photoProvider.DeletePhoto("C:\\test\\test.jpg");
            }

            container.RegisterInstance(photoProvider);

            ModelContext.Initialize(container);

            Photo.DeletePhoto("C:\\test\\test.jpg");
            
            repository.VerifyAll();
        }

        /// <summary>
        /// Deletes a photo by using the path to the photo
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DeletePhotoDirectWithoutPath()
        {
            MockRepository repository = new MockRepository();
            IPhotoDataProvider photoProvider = null;
            IUnityContainer container = new UnityContainer();

            using (repository.Record())
            {
                photoProvider = repository.StrictMock<IPhotoDataProvider>();
                photoProvider.DeletePhoto("C:\\test\\test.jpg");
            }

            container.RegisterInstance(photoProvider);

            ModelContext.Initialize(container);

            Photo.DeletePhoto(null);
        }
    }
}
